#Pico Clock by Adrian Rawson 'ahrprojects.co.uk' 2023 Version 1.3.
#A 3D printed hollow clock design with RPi Pico-w control program.
#The clock gets its time from the internet (ntp server).
#The hands are automatically synchronised after a switch off.
#A local web page provides various clock controls.
#A neopixel ring displays location of seconds, minutes and hours.
#Ambiant temperature is also displayed on neopixel ring.
#Neopixel intensity is controlled by a light sensor.
#A low light mode illuminates the hands and 5 minute marks.

import array, time
from machine import Pin, PWM, I2C, SPI, ADC, RTC, Timer
import ssd1306
import rp2
import random
from utime import sleep
import network
import socket
from oled import Write
from oled.fonts import ubuntu_mono_15, ubuntu_mono_20
import os
import ntptime
import uasyncio as asyncio
import _thread
import onewire
import ds18x20
import struct
import gc

#WiFi Details
SSID="SSID"
PASSWORD="PASSWORD"

# Configure the number of WS2812 LEDs, pins and brightness.
NUM_LEDS = 60
brightness = 0.1 #minimum 0.004
BLACK = (0, 0, 0)
RED = (255, 0, 0)
YELLOW = (255, 150, 0)
ORANGE = (255, 80, 0)
GREEN = (0, 255, 0)
PINK = (255,128,128)
CYAN = (0, 255, 255)
BLUE = (0, 0, 255)
PURPLE = (180, 0, 255)
WHITE = (255, 255, 255)
COLORS = (BLACK, RED, YELLOW, ORANGE, GREEN, CYAN, BLUE, PURPLE, WHITE)

mcolor=PURPLE
hcolor=CYAN
scolor=WHITE
tcolor=BLUE
ncolor=GREEN
ds_pin = Pin(27)
ds_sensor = ds18x20.DS18X20(onewire.OneWire(ds_pin))
roms=ds_sensor.scan()
synchronised=False
flash=True
led = PWM(Pin(17))
led.freq(1000)
led.duty_u16(0)
BST = 1
buzzer = PWM(Pin(15))
tim=Timer()
spi =  SPI(1, 10_000_000, sck=Pin(14), mosi=Pin(15), miso=Pin(8))
dc = Pin(12)   # data/command
rst2 = Pin(9)
rst3 = Pin(13)
cs2 = Pin(10)
cs3 = Pin(11)

oled2 = ssd1306.SSD1306_SPI(128, 64, spi,dc,rst2,cs2)
oled3 = ssd1306.SSD1306_SPI(128, 64, spi,dc,rst3,cs3)

light_value = ADC(28)

wlan = network.WLAN(network.STA_IF)

mpins = [Pin(3,Pin.OUT),Pin(2,Pin.OUT),Pin(1,Pin.OUT),Pin(0,Pin.OUT)]
hpins = [Pin(7,Pin.OUT),Pin(6,Pin.OUT),Pin(5,Pin.OUT),Pin(4,Pin.OUT)]
full_step_ccw = [[1,0,0,0],[0,1,0,0],[0,0,1,0],[0,0,0,1]]
full_step_cw = [[0,0,0,1],[0,0,1,0],[0,1,0,0],[1,0,0,0]]
#half_step_cw = [[0,0,0,1],[0,0,1,1],[0,0,1,0],[0,1,1,0],[0,1,0,0],[1,1,0,0],[1,0,0,0],[1,0,0,1]]
mdelay=500 #the delay in microsecs for motor steps

oled2.fill(0)
oled3.fill(0)

@rp2.asm_pio(sideset_init=rp2.PIO.OUT_LOW, out_shiftdir=rp2.PIO.SHIFT_LEFT, autopull=True, pull_thresh=24)
def ws2812():
    T1 = 2
    T2 = 5
    T3 = 3
    wrap_target()
    label("bitloop")
    out(x, 1)               .side(0)    [T3 - 1]
    jmp(not_x, "do_zero")   .side(1)    [T1 - 1]
    jmp("bitloop")          .side(1)    [T2 - 1]
    label("do_zero")
    nop()                   .side(0)    [T2 - 1]
    wrap()
 
# Create the StateMachine with the ws2812 program, outputting on Pin(16).
sm0 = rp2.StateMachine(0, ws2812, freq=8_000_000, sideset_base=Pin(16)) 
# Start the StateMachine, it will wait for data on its FIFO.
sm0.active(1) 
# Display a pattern on the LEDs via an array of LED RGB values.
ar = array.array("I", [0 for _ in range(NUM_LEDS)])

fcolors=[PURPLE,CYAN,WHITE,BLUE,GREEN]
def set_light_mode(value):
    #lightmode values:
    #0 = No lights
    #1 = Seconds only
    #2 = Seconds, Minutes, Hours
    #3 = Seconds, Minutes, Hours, temperature
    #4 = Temperature only
    #5 = Night mode with full illumination
    #6 = Night mode hands only
    global mcolor,hcolor,scolor,tcolor,ncolor
    if value==0:
        mcolor=BLACK
        hcolor=BLACK
        scolor=BLACK
        tcolor=BLACK
        ncolor=BLACK
    elif value==1:
        mcolor=BLACK
        hcolor=BLACK
        scolor=fcolors[2]
        tcolor=BLACK
        ncolor=fcolors[4]
    elif value==2:
        mcolor=fcolors[0]
        hcolor=fcolors[1]
        scolor=fcolors[2]
        tcolor=BLACK
        ncolor=fcolors[4]
    elif value==3:
        mcolor=fcolors[0]
        hcolor=fcolors[1]
        scolor=fcolors[2]
        tcolor=fcolors[3]
        ncolor=fcolors[4]
    elif value==4:
        mcolor=BLACK
        hcolor=BLACK
        scolor=BLACK
        tcolor=fcolors[3]
        ncolor=fcolors[4]  
dim=0
def get_light_value():
    #manipulate brightness of displays and neopixels according to light level
    global brightness,dim
    intensity=light_value.read_u16()
    #print (intensity)
    if intensity > 50000:
        dim=0
        brightness=1
        oled2.contrast(255)
        oled3.contrast(255)
    elif intensity > 40000 and intensity <50000:
        dim=1
        brightness=0.5
        oled2.contrast(200)
        oled3.contrast(200)
    elif intensity > 30000 and intensity <40000:
        dim=2
        brightness=0.1
        oled2.contrast(100)
        oled3.contrast(100)
    elif intensity > 20000 and intensity <30000:
        dim=3
        brightness=0.05
    elif intensity > 10000 and intensity <20000:
        dim=4
        brightness=0.01
        oled2.contrast(0)
        oled3.contrast(0)
    elif intensity < 10000:
        dim=5
        brightness=0.008
        oled2.contrast(0)
        oled3.contrast(0)

def pixels_show(brightness):
    #display current state of neopixels
    dimmer_ar = array.array("I", [0 for _ in range(NUM_LEDS)])
    for i,c in enumerate(ar):
        r = int(((c >> 8) & 0xFF) * brightness)
        g = int(((c >> 16) & 0xFF) * brightness)
        b = int((c & 0xFF) * brightness)
        dimmer_ar[i] = (g<<16) + (r<<8) + b
    sm0.put(dimmer_ar, 8)
    time.sleep_ms(10)
        
def pixels_set(i, color):
    #set neopixel colour
    if i>=NeoOffset:
        ar[i-60] = (color[1]<<16) + (color[0]<<8) + color[2]
    else:
         ar[i] = (color[1]<<16) + (color[0]<<8) + color[2]
         
def pixels_fill(color):
    #not used
    for i in range(len(ar)):
        pixels_set(i, color)

temperature=0
def get_temp():
    #read the onewire temperature sensor and set value
    global temperature,tcolor
    ds_sensor.convert_temp()
    time.sleep(0.05)
    for rom in roms:
        temperature=round(ds_sensor.read_temp(rom),1)
    if synchronised:
        mprint(str(temperature)+" C",20,40,2,oled2)
    if tcolor!=BLACK:     
        if int(temperature) <=0:
            tcolor=BLUE
        if int(temperature) >0:
            tcolor=YELLOW
        if int(temperature) >10:
            tcolor=ORANGE
        if int(temperature) >20:
            tcolor=RED

currentt=0
def set_temp(loc):
    #show temperature on neopixel ring
    global currentt
    pixels_set(currentt,BLACK)
    if flash:
        pixels_set(loc,tcolor)
    else:
        pixels_set(loc,BLACK)
    currentt=loc
    
currenth=0        
def set_hour(loc):
    #set neopixel for hour hand
    global currenth
    if loc!= currenth:
        pixels_set(currenth,BLACK)
        pixels_set(loc,hcolor)
        currenth=loc
    
currentm=-1
#This calls motor moves via core 2 thread for minute and hour hand
#Hour hand nudges every 12 minutes
def set_min(loc):
    #set neopixel for minute hand and call synchronisation
    global currentm, synchronised
    if loc!= currentm and currentm >=0:
        pixels_set(currentm,BLACK)
        pixels_set(loc,mcolor)
        if loc-NeoOffset==0 or (loc-NeoOffset) % 12==0:
            hourmove=1
        else:
            hourmove=0
        
        if synchronised:
            _thread.start_new_thread(step_hands,(1,full_step_cw,hourmove,full_step_cw,))
            write_hands(loc,currenth)
        else:
            currentm=loc
            set_hands()
    currentm=loc
    
currents=0    
def set_sec(loc):
    #set neopixel for second
    global currents,flash
    pixels_set(currents,BLACK)
    pixels_show(brightness)
    pixels_set(currentm,mcolor)
    pixels_set(currenth,hcolor)
    if dim > 4:
        set_luminous(True)
    else:
        set_luminous(False)
    pixels_set(loc,scolor)
    pixels_show(brightness)
    if loc % 2 == 0:
        flash = False
    else:
        flash = True
    currents=loc

luminous=[0,5,10,15,20,25,30,35,40,45,50,55]
def set_luminous(on):
    #turn on 5 minute neopixels for night time display
    #turn off oleds
    if on:
        color=ncolor
        #this sets the led on at brightness 30%
        led.duty_u16(int(65025*0.5))
        oled2.poweroff()
        oled3.poweroff()
    else:
        color=BLACK
        led.duty_u16(0)
        oled2.poweron()
        oled3.poweron()
    for i in luminous:
        pixels_set(i,color)

    pixels_set(currentm,mcolor)
    pixels_set(currenth,hcolor)
    set_temp(currentt)
        
said=["O'clock","1 minute","2 minutes","3 minutes","4 minutes","Five",
      "6 minutes","7 minutes","8 minutes","9 minutes","Ten",
      "11 minutes","12 minutes","13 minutes","14 minutes","A Quarter",
      "16 minutes","17 minutes","18 minutes","19 minutes","Twenty",
      "21 minutes","22 minutes","23 minutes","24 minutes","Twenty-five",
      "26 minutes","27 minutes","28 minutes","29 minutes","Half"]
hour=['One','Two','Three','Four','Five','Six','Seven','Eight','Nine','Ten','Eleven','Twelve']

def daystring(td):
    #create english time of day string
    if td[3]+BST > 11:
        th=td[3]+BST-12
    else:
        th=td[3]+BST
    timestring20=''
    timestring40=''
    if td[4] <= 30 and td[4] > 0:
        timestring20=((said[td[4]]))
        timestring40=(("past "+hour[th-1]))
    else:
        if td[4] == 0:
            timestring20=(hour[th-1])
            timestring40=(said[0])
        else:
            tm=td[4]-((td[4]-30)*2)
            timestring20=((said[tm]))
            if th==12:
                timestring40=(("to "+hour[0]))
            else:
                timestring40=(("to "+hour[th]))
    if td[4]+NeoOffset != currentm:
        mprint(timestring20,0,20,2,oled3)
        mprint(timestring40,0,40,2,oled3)
NeoOffset=30;
day=['Mon','Tue','Wed','Thu','Fri','Sat','Sun']
month=['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']

timezone=0
currentd=-1
def get_time():
    #read the real time clock and set clock displays accordingly
    global lasts,currentd
    now=time.time()
    td=time.localtime(now)
    th=(td[3]+BST)
    tm=(td[4])
    ts=(td[5])
    tz=th+timezones[timezone][1]
    if tz>23:
        tz=tz-24
    if tz<0:
        tz=tz+23
    stime=timezones[timezone][0]+" "'{:2}'.format(str(tz))+':'+'{:2}'.format(str(tm)+':'+'{:2}'.format(str(ts)))
    sdate='{:#1}'.format(str(td[2]))+'/'+'{:#1}'.format(str(td[1]))+'/'+'{:#1}'.format(str(td[0]))
    wdate='{:3}'.format(day[td[6]])+'/'+'{:#1}'.format(str(td[2]))+'/'+'{:3}'.format(month[td[1]-1])
    ydate=str(td[0])
    if synchronised:
        if td[6] != currentd:
            mprint(wdate,10,0,2,oled2)
            mprint(ydate,30,20,2,oled2)
            currentd = td[6]
    else:
        mprint("synchronising",0,48,1,oled2)
    daystring(td)
    if th > 11:
        th=th-12
    hl=(th)*5+int(tm/12)#hours adjusted to map onto 60 lamps
    set_hour(hl+NeoOffset)
    set_min(tm+NeoOffset)
    set_sec(ts+NeoOffset)
    set_temp(int(round(temperature,0))+NeoOffset)
    mprint(stime,0,0,2,oled3)

def buzz():
    #experimental - not used
    print('buzz')
    buzzer.freq(6000)
    buzzer.duty_u16(30000)
    time.sleep(0.5)
    buzzer.duty_u16(0)
    time.sleep(0.5)
    buzzer.freq(4000)
    buzzer.duty_u16(30000)
    time.sleep(0.5)
    buzzer.duty_u16(0)
    
def mprint(text,x,y,s,oled):
    #write test to oleds
    if s==1:
        write15 = Write(oled,ubuntu_mono_15)
        write15.text('0123456789012345',x,y,0)
        write15.text(text,x,y,1)
    elif s==2:
        write20 = Write(oled,ubuntu_mono_20)
        write20.text('0123456789012',x,y,0)
        write20.text(text,x,y,1)
    oled.show()
    
def set_time():
    #set the time for real time clock from intenet NTP server
    NTP_DELTA = 2208988800
    host = "sntp.cam.ac.uk"
    NTP_QUERY = bytearray(48)
    NTP_QUERY[0] = 0x1B
    addr = socket.getaddrinfo(host, 123)[0][-1]
    s=socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.settimeout(2)
        res = s.sendto(NTP_QUERY, addr)
        msg = s.recv(48)
    finally:
        s.close()
    val = struct.unpack("!I", msg[40:44])[0]
    t = val - NTP_DELTA
    tm = time.gmtime(t)
    RTC().datetime((tm[0], tm[1], tm[2], tm[6]+1, tm[3], tm[4], tm[5], 0))

timeset=False
def connect_to_network():
    #connect to WiFi
    global timeset
    wlan.active(True)
    wlan.config(pm = 0xa11140)  # Disable power-save mode
    wlan.connect(SSID, PASSWORD)
    max_wait = 30
    max_try = 10
    while max_wait > 0:
        if wlan.status() ==3: #< 0 or wlan.status() >= 3:
            break
        max_wait -= 1
        mprint(str(max_wait)+" ",0,0,1,oled2)
        time.sleep(1)

    if wlan.status() != 3:
        mprint('Connect fail',0,16,1,oled2)
    else:
        mprint('Connected',0,0,1,oled2)
        status = wlan.ifconfig()
        mprint('ip ' + status[0],0,16,1,oled2)
        mprint('PYBD08DA.local',0,32,1,oled2)
        try:
            set_time()
            #ntptime.settime()
            #force_time()
            now=time.localtime()
            mprint(str(now[2])+"/"+str(now[1])+"/"+str(now[0])+" "+str(now[3])+":"+str(now[4]),0,48,1,oled2)
            timeset=True
        except:
            mprint("settime fail",0,32,1,oled2)
            
timezones=[["UK",0],["AUS",9],["NZ",11],["PDT",-7],["CDT",-5]]

def webpage():
    #create web page for client connections
    html = f"""
            <!DOCTYPE html>
            <html>
            <h1>Pico Clock Control</h1>
            <hr>
            <form action="./showlights">
            <label for="slights">Show lights</label><br>
            <select id="lights" name="lights">
            <option value="0">All Off</option>
            <option value="1">Seconds</option>
            <option value="2">H,M,S</option>
            <option value="3">H,M,S,Temp</option>
            <option value="4">Temp Only</option>
            </select>
            <input type="submit" value="Submit" /><br>
            </form>
            <hr>
            <p> 
            The minute and hour nudge controls have a maximum nudge value of one sixtieth of the clock face, so
            in the case of a minute hand nudge a value of 64 steps would nudge half a minute clockwise.</p>

            <form action="./nudgeminutes">
            <label for="stepmcw">Step minute hand clockwise -128 to 128 steps</label><br>
            <label for="nsteps">&nbsp;&nbsp;Number of steps:</label>
            <input type="text" id="param1" name="param1" value="0" size=4>
            <input type="submit" value="Submit" /><br><br>
            </form>

            <form action="./nudgehours">
            <label for="stephcw">Step hour hand clockwise -64 to 64 steps</label><br>
            <label for="nsteps">&nbsp;&nbsp;Number of steps:</label>
            <input type="text" id="param1" name="param1" value="0" size=3>
            <input type="submit" value="Submit" /><br>
            </form>
            <hr>
            <p>
            The hands should synchronise when the second minute of run time starts. However if they don't
            their known values can be entered with the 'set hands location' control, before disconnecting 
            and reconnecting the power. Hand locations are defined by minute number (e.g. 8 o'clock is 40)
            </p>            
            <form action="./writehands">
            <label for="sethands">Set hands location</label><br>
            <label for="minutes">&nbsp;&nbsp;Minute hand location: </label>
            <input type="text" id="param1" name="param1" value="0" size=2><br>
            <label for="hours">&nbsp;&nbsp;Hour hand location: </label>
            <input type="text" id="param2" name="param2" value="0" size=2>
            <input type="submit" value="Submit" /><br>
            </form>            
            <hr>
            <form action="./bst">
            <label for="bst">Set BST: 0 for Off, 1 for On</label><br>
            <input type="text" id="param1" name="param1" value="0" size=1>
            <input type="submit" value="Submit" /><br>
            <hr>
            </form>
            <p>This timezone setting only affects the 24 hour clock display</p>
            <form action="./timezone">
            <label for="tz">Set timezone</label><br>
            <select id="tz" name="tz">
            <option value="0">{timezones[0][0]}</option>
            <option value="1">{timezones[1][0]}</option>
            <option value="2">{timezones[2][0]}</option>
            <option value="3">{timezones[3][0]}</option>
            <option value="4">{timezones[4][0]}</option>
            </select>
            <input type="submit" value="Submit" /><br>
            </form>
            <hr>
            </body>
            </html>
            """
    return str(html)

def pars_message(request):
    #extract values from client request
    try:
        message=request.split()
        param1=message[1].split('?')
        if "&" in param1[1]:
            param2=param1[1].split('&')
            data1=param2[0].split('=')
            data2=param2[1].split('=')
            value1=data1[1]
            value2=data2[1]
        else:
            value2='0'           
            data=param1[1].split('=')
            value1=data[1]
        return int(value1),int(value2)
    except:
        return 0,0

async def serve_client(reader, writer):
    global BST,timezone
    #listen for client requests   
    request_line = await reader.readline()
    # We are not interested in HTTP request headers, skip them
    while await reader.readline() != b"\r\n":
        pass
    request = str(request_line)
    html = webpage()
    writer.write(html)
    
    readings = request.find('')
    showlights = request.find('/showlights')
    writehands = request.find('/writehands')
    nudgeminutes = request.find('/nudgeminutes')
    nudgehours = request.find('/nudgehours')
    bst = request.find('/bst')
    tz = request.find('/timezone')

    value,value2 = pars_message(request)
        
    if writehands == 6:
        write_hands_file(value,value2)
        stateis = "Hands Updated to "+str(value)+","+str(value2)
    elif nudgeminutes == 6:
        if value >=129:
            value=128
        if value < -128:
            value=-128
        if value <0:
            nudge_hands(mpins,full_step_ccw,abs(value),1)
        else:
            nudge_hands(mpins,full_step_cw,value,1) 
        stateis = "nudged minutes clockwise by "+str(value)
    elif nudgehours == 6:
        if value >=65:
            value=64
        if value < -64:
            value=-64
        if value <0:
            nudge_hands(hpins,full_step_ccw,abs(value),1)
        else:
            nudge_hands(hpins,full_step_cw,value,1)
        stateis = "nudged hours clockwise by "+str(value)
    elif showlights == 6:
        if value >4:
            stateis = "Show lights set to unrecognised value"
        else:
            set_light_mode(value)
            stateis = "Show lights set to "+str(value)
    elif bst == 6:
        if value >1:
            stateis = "BST unrecognised value"
        else:
            if value<BST: #reverse
                #nudge_hands(hpins,full_step_ccw,64,5)
                _thread.start_new_thread(step_hands,(0,full_step_cw,5,full_step_ccw))
            elif value>BST:
                #nudge_hands(hpins,full_step_cw,64,5)
                _thread.start_new_thread(step_hands,(0,full_step_cw,5,full_step_cw))
            BST=value
            stateis = "BST set to "+str(value)
    elif tz == 6:
        timezone=value
        stateis = "timezone set to "+timezones[value][0]
    else:
        stateis = "Unknown Request"
    response = stateis
    writer.write(response)

    await writer.drain()
    await writer.wait_closed() 
        
def step_hands(mcycles,mdirection,hcycles,hdirection):
    #called via thread to run in core one
    for c in range(mcycles):
        for n in range(128):
            for step in mdirection:
                for i in range(len(mpins)):
                    mpins[i].value(step[i])
                    time.sleep_us(mdelay)

    for i in range(len(mpins)):
        mpins[i].value(0)
        
    if hcycles>0:
        for c in range(hcycles):
            for n in range(64):
                for step in hdirection:
                    for i in range(len(hpins)):
                        hpins[i].value(step[i])
                        time.sleep_us(mdelay)
        for i in range(len(hpins)):
            hpins[i].value(0)

def nudge_hands(pins,direction,amount,cycles):
    #cycles always one when called
    for c in range(cycles):
        for n in range(amount):
            for step in direction:
                for i in range(len(pins)):
                    pins[i].value(step[i])
                    time.sleep_us(mdelay)
    for i in range(len(pins)):
        pins[i].value(0)                   
                
def write_hands(m,h):
    #print(m,h)
    #called every minute to store current location of hands
    file=open("timelast.txt","w")
    m=m-NeoOffset
    h=h-NeoOffset
    file.write(str(m)+"\n")#this stores actual minutes from Neo zero
    file.write(str(h)+"\n")#this stores hour hand minute value from Neo Zero
    file.write(str(BST))#this stores the current BST value
    file.close()

def get_direction(delta):
    #calculate best direction to achieve hand move
    if delta<0:
        cw = False #ccw
    else:
        cw = True #cw
    if abs(delta)>30:
        delta=60-abs(delta)
        cw = not cw #reverse direction
    if cw:
        direction=full_step_cw
    else:
        direction=full_step_ccw
    return abs(delta),direction
    
def set_hands():
    # hand synchronisation after first minute from switch on
    global synchronised,BST
    file=open("timelast.txt","r")
    lastm=file.readline()
    lasth=(file.readline())
    bst=(file.readline())
    BST=int(bst.strip())
    file.close()
    now=time.time()
    td=time.localtime(now)
    th=(td[3]+BST)
    tm=(td[4])
    if th>12:
        th=th-12
    hl=(th)*5+int(tm/12)#hours adjusted to map onto 60 lamps
    deltam=tm-int(lastm.strip())
    mcycles=get_direction(deltam)[0]
    mdirection=get_direction(deltam)[1]
    deltah=hl-int(lasth.strip())
    hcycles=get_direction(deltah)[0]
    hdirection=get_direction(deltah)[1]
    _thread.start_new_thread(step_hands,(mcycles,mdirection,hcycles,hdirection,))
    oled2.fill(0)
    write_hands(currentm,currenth)
    synchronised=True

def write_hands_file(m,h):
    #insert new values in timelast,txt
    file=open("timelast.txt","w")
    file.write(str(m)+"\n")#this stores actual minutes from Neo zero
    file.write(str(h)+"\n")#this stores hour hand minute value from Neo Zero
    file.write(str(BST))#this stores the current BST value
    file.close()
    
def force_time():
    #debug routine - not used
    global timeset
    RTC().datetime((2023, 5, 3, 3, 10, 58, 52, 0))
    timeset=True
    
def onesec(t):
    #print("1 sec")
    get_time()
    
#write_hands_file(14,21)         
#nudge_hands(mpins,full_step_cw,20,1)

async def main():
    connect_to_network()
    #print('Setting up webserver...')
    asyncio.create_task(asyncio.start_server(serve_client, "0.0.0.0", 80))
    if timeset:
        while True:
            get_time()
            get_temp()
            get_light_value()
            gc.collect()
            await asyncio.sleep(0.6)
try:
    asyncio.run(main())
finally:
    asyncio.new_event_loop()